/*
History:

First version was only used to compress the DreaMon Reset Logo, but
it turned out to be useful for other projects too.

1.1
* screen and color data are now expected boardpainter format in a *.P00 file

1.0

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int dumpBuf(unsigned char *buf, int len, FILE *outFile);

int main(int argc, char * argv[])
{
	// logo area start and end
	unsigned char logoy0, logoy1;
	// uncompressed logo (and color) size
	unsigned int logoRawSize;
	// compressed logo and color size
	unsigned int scrCompSize, colCompSize;
	// the filenames for screendata,
	char *scrName, *outName;
	// filehandles for above
	FILE *scrFile, *outFile;
	// buffers for screen and color data
	unsigned char *scrBuf, *colBuf;
	// compressed screen and colors
	unsigned char *scrCom,  *colCom;
	// tilebuffer
	unsigned char tiles[16];
	// colors
	unsigned char colors[4];

	unsigned char *cc, *ce;
	int cnt0, cnt1, tile_cnt, color_cnt;


	logoy0 = 0;
	logoy1 = 3;
	logoRawSize = 40*(logoy1-logoy0+1);
	scrCompSize = logoRawSize/2;
	colCompSize = logoRawSize/4;

	scrName = outName = NULL;
	scrFile = outFile = NULL;
	scrBuf = colBuf = NULL;
	scrCom = colCom = NULL;

	printf("Blocky Logo compressor V1.1 by Doc Bacardi/DRM.\n");

	if(argc!=3)
	{
		fprintf(stderr, "Syntax: %s <scrFile> <outFile>\n\n",argv[0]);
		exit(1);
	}

	scrName=argv[1];
	outName=argv[2];

	// open input and output files
	if( !(scrFile=fopen(scrName,"rb")) )
	{
		fprintf(stderr, "Unable to open screencode File!\n");
		goto err_exit;
	}
	if( !(outFile=fopen(outName,"w")) )
	{
		fprintf(stderr, "Unable to open output File!\n");
		goto err_exit;
	}

	// allocate mem for the input and ouptut buffers
	if(
		(scrBuf=(unsigned char*)malloc(logoRawSize))==NULL ||
		(colBuf=(unsigned char*)malloc(logoRawSize))==NULL ||
		(scrCom=(unsigned char*)malloc(scrCompSize))==NULL ||
		(colCom=(unsigned char*)malloc(colCompSize))==NULL
	  )
	{
		fprintf(stderr, "Unable to open output File!\n");
		goto err_exit;
	}

	// clear the buffers (just nice for debugging, does not affect output data)
	memset(scrCom,0,scrCompSize);
	memset(colCom,0,colCompSize);

	//Ok, all files are open and buffers allocated. Read in data now.
	if( fseek(scrFile,0x1c,SEEK_SET)!=0 )
	{
		fprintf(stderr, "Can't seek to screendata!\n");
		goto err_exit;
	}
	if( fread(scrBuf,1,logoRawSize,scrFile)!=logoRawSize)
	{
		fprintf(stderr, "Read Error!\n");
		goto err_exit;
	}
	if( fseek(scrFile,0x41c,SEEK_SET)!=0 )
	{
		fprintf(stderr, "Can't seek to screendata!\n");
		goto err_exit;
	}
	if( fread(colBuf,1,logoRawSize,scrFile)!=logoRawSize)
	{
		fprintf(stderr, "Read Error!\n");
		goto err_exit;
	}

	fclose(scrFile);
	scrFile=NULL;

	// clear upper nibble of color data (contains random data)
	for(ce=(cc=colBuf)+logoRawSize; cc<ce; ++cc)
	{
		*cc &= 0x0f;
	}

	tile_cnt=0;
	for(cnt0=0; cnt0<logoRawSize; ++cnt0)
	{
		for(cnt1=0; cnt1<tile_cnt && scrBuf[cnt0]!=tiles[cnt1]; cnt1++);
		if( cnt1==tile_cnt )
		{
			if( tile_cnt==0x10 )
			{
				fprintf(stderr, "More than 16 tiles in screenfile!\n");
				goto err_exit;
			}
			tiles[tile_cnt++]=scrBuf[cnt0];
		}
		scrCom[cnt0>>1] <<= 4;
		scrCom[cnt0>>1]  |= cnt1;
	}

	// count used colors
	color_cnt=0;
	for(cnt0=0; cnt0<logoRawSize; ++cnt0)
	{
		for(cnt1=0; cnt1<color_cnt && colBuf[cnt0]!=colors[cnt1]; cnt1++);
		if( cnt1==color_cnt )
		{
			if( color_cnt==4 )
			{
				fprintf(stderr, "More than 4 colors in logo!\n");
				goto err_exit;
			}
			colors[color_cnt++]=colBuf[cnt0];
		}
		colCom[cnt0>>2] <<= 2;
		colCom[cnt0>>2]  |= cnt1;
	}


	fprintf(outFile,";---------------------------------------\n");
	fprintf(outFile,"; Compressed Logo\n");
	fprintf(outFile,";---------------------------------------\n");

	fprintf(outFile,"\nLogo_Scr:\n");
	dumpBuf(scrCom, scrCompSize, outFile);

	fprintf(outFile,"\nLogo_Col:\n");
	dumpBuf(colCom, colCompSize, outFile);

	fprintf(outFile,"\nLogo_ScrTiles:\n");
	dumpBuf(tiles, tile_cnt, outFile);

	fprintf(outFile,"\nLogo_ColTiles:\n");
	dumpBuf(colors, color_cnt, outFile);

	fclose(outFile);
	free(scrBuf);
	free(colBuf);
	free(scrCom);
	free(colCom);

	exit(0);

err_exit:
	if( scrFile )
		fclose(scrFile);
	if( outFile )
		fclose(outFile);

	if( scrBuf!=NULL )
		free(scrBuf);
	if( colBuf!=NULL )
		free(colBuf);
	if( scrCom!=NULL )
		free(scrCom);
	if( colCom!=NULL )
		free(colCom);

	exit(1);
}


int dumpBuf(unsigned char *buf, int len, FILE *outFile)
{
	int cnt0;


	for(cnt0=0; cnt0<len; cnt0++)
	{
		if( (cnt0&7)==0 )
			fprintf(outFile,"\t.DB ");
		fprintf(outFile,"$%02X%c",buf[cnt0],((cnt0&7)!=7 && cnt0!=len-1 ? ',' : '\n'));
	}
	return 0;
}

